import { Component, Input, SimpleChanges } from '@angular/core'
import { ActionsService } from './actions.service'
import { FileService } from '../../../file.service'
import { AuthService } from 'src/app/auth/auth.service'
import { MatSnackBar } from "@angular/material/snack-bar";
import { FileSaverService } from 'ngx-filesaver';


@Component({
  selector: 'app-actions-card',
  templateUrl: './actions-card.component.html',
  styleUrls: ['./actions-card.component.scss']
})
export class ActionsCardComponent {
  public analyzing: boolean = false
  _dataSource
  favorited

  @Input() 
  set dataSource(dataSource){
      this._dataSource = dataSource
  }

  constructor(private actionsService: ActionsService, public fileService: FileService, private authService: AuthService, private _snackBar: MatSnackBar, private FileSaverService: FileSaverService){}

  onDelete() { 
    // TODO
  }

  onEdit() {
   // TODO
  }
  
  onFavorite() {
    this.favorited = !this.favorited
    if(this.favorited) {
      // set favorite with selecte hash
      this.actionsService.setFavorite(this._dataSource.sha256)
        .subscribe((data) => {
          this.openSnackBar(`Added to favorites`)
          // get profile afterwards to reflect change
          this.authService.getUserProfile()
          }, error => {
              this.openSnackBar(`ERROR`)
          })

    } else {
      // remove favorite with selecte hash
      this.actionsService.setFavorite(this._dataSource.sha256)
        .subscribe((data) => {
          this.openSnackBar(`Removed from favorites`)
          }, error => {
              this.openSnackBar(`ERROR`)
          })
      
    }
  }

  onReAnalyze() {

  }

  // listen for changes and change color of the button if there's a new value
  ngOnChanges() {
    if(this._dataSource) {
      if(this.authService.user.favorites.includes(this._dataSource.sha256)) {
        this.favorited = true
      } else {
        this.favorited = false
      }
    }
  }

  // download file if user lcick download button
  onDownload() {
    this.actionsService.downloadFile(this._dataSource.sha256)
    .subscribe(response => {
      this.openSnackBar(`Downloading`)
      // pass data to filesaverservice
      this.FileSaverService.save(response, this._dataSource.sha256);
    },
    error => {
      console.log(error)
      this.openSnackBar(`An error occured.`)
    })
  }

  ngOnInit(){
    
  }

  // send file to reanalyze
  reanalyze( source: string ) {
    console.log("Sending to analyze", this._dataSource.sha256)
    this.fileService.reanalyze(this._dataSource.sha256, source)
      .subscribe((data) => {
        this.analyzing = true
        }, error => {
          console.log(error)
        })
    }


    openSnackBar(message: string) {
      this._snackBar.open(message, 'Dismiss', {
        duration:2000
      })
    }

}
