import { COMMA, ENTER } from '@angular/cdk/keycodes';
import { Component, Input } from '@angular/core';
import { MatChipInputEvent } from '@angular/material/chips';
import { MatSnackBar } from '@angular/material/snack-bar';
import { AuthService } from 'src/app/auth/auth.service';
import { ActionsService } from '../actions-card/actions.service';

@Component({
  selector: 'app-tags-card',
  templateUrl: './tags-card.component.html',
  styleUrls: ['./tags-card.component.scss']
})

export class TagsCardComponent {
  _dataSource
  favorited
  permission = false


  @Input() 
  set dataSource(dataSource){
      this._dataSource = dataSource
  }

  constructor(private _snackBar: MatSnackBar, private actionsservice: ActionsService, private AuthService: AuthService) {}

  readonly separatorKeysCodes: number[] = [ENTER, COMMA];
  tags = [];
  user
  add(event: MatChipInputEvent): void {
    const input = event.input;
    const value = event.value;
    // Add our fruit
    if ((value || '').trim()) {
      this.tags.push(event.value);
      this.actionsservice.setTags(this._dataSource.sha256, this.tags)
      .subscribe((data) => {

        this.openSnackBar(`Updated tags`)
        }, error => {
            this.openSnackBar(`ERROR`)
      })
    }

    // Reset the input value
    if (input) {
      input.value = '';
    }

    
  }

  // remove tag
  remove(tag : string): void {
    const index = this.tags.indexOf(tag);

    if (index >= 0) {
      // remove element from array 
      this.tags.splice(index, 1);

      // set new tags
      this.actionsservice.setTags(this._dataSource.sha256, this.tags)
      .subscribe((data) => {
        this.openSnackBar(`Updated tags`)
        }, error => {
            this.openSnackBar(`ERROR`)
      })
    }
  }

  ngOnInit(){
    this.user = this.AuthService.user
  }

  // listen for changes
  ngOnChanges() {
    console.log(this._dataSource)
    console.log(this.user)
    if(this._dataSource && this.user) {
      this.tags = this._dataSource.tags
      if(this.user.name == this._dataSource.uploaded_by || this.user.role == 'admin' || this.user.role == 'researcher') {
        this.permission = true
      }
    }
    
  }

  openSnackBar(message: string) {
    this._snackBar.open(message, 'Dismiss', {
      duration:2000
    })
  }
}