import { Injectable } from '@angular/core';
import { HttpClient, HttpParams } from '@angular/common/http'
import { File } from '../dashboard/file/file-list/file-list.model'
import { Observable, Subject } from 'rxjs'
import { FileData } from '../dashboard/file/file-list/file-list.model'

@Injectable({ providedIn: "root" })

export class FavoriteService {
    private favoritesUpdated = new Subject<Boolean>()
    constructor(private http: HttpClient) {}
    
    // prepare and send http request to get favorites
    getFavoriteFiles(filesPerPage, pageIndex) {
        const paramData = {size: filesPerPage, index: pageIndex};
        return this.http.get<{message: String, files: File[]}>("/api/v1/user/favorite", {params: paramData})
    }
    
    // return favorite observable
    getFavoritesUpdateListner() {
        return this.favoritesUpdated.asObservable()
    }

    // get number of favorites
    getFavoriteCount() {
        return this.http.get<{count: number}>("/api/v1/user/favorite/count")
    }
    
    // get file
    getFile( sha256: string ): Observable<any> {
        let params = new HttpParams().set('sha256', sha256)
        return this.http.get(`/api/v1/file/${sha256}`, { params: params }) 
    }
}