swagger: '2.0'
info:
  description: |
    A Malware and Goodware repository.
  version: 1.0.0
  title: Malware and Goodware Repository
  
tags:
- name: File
  description: All routes conserning files
- name: User
  description: All routes conserining users
# schemes:
# - http
paths:
  /api/v1/file/:
    get:
      tags:
      - File
      summary: Gets a set of files from the database
      description: Used for getting a specific number of files and searching in the database.
      produces:
      - application/json
      parameters:
      - name: size
        in: query
        description: The paginate size to get
        required: false
        type: string
      - name: index
        in: query
        description: The paginate index to start from
        required: false
        type: string
      - name: text
        in: query
        description: The string to search for in the database
        required: false
        type: string
      responses:
        200:
          description: Successful
          schema:
            type: array
            items:
              $ref: '#/definitions/file'
        500:
          description: If the database throws any error
  
  /api/v1/file/{hash}:
    get:
      tags:
      - File
      summary: Get data for a specific file
      description: Used for grabbing metadata and analysis-date of a sample stored in the database
      produces:
      - application/json
      parameters:
      - name: hash
        required: true
        in: path
        description: The hash of the requested file
        type: string

      responses:
        200:
          description: Successful
          schema:
            $ref: '#/definitions/file'
            
        404:
          description: If the file does not exist in the database
        500:
          description: If the database throws any error
          
  /api/v1/file/{hash}/file:
    get:
      tags:
      - File
      summary: Downloads a file
      description: Triggers a download of a specific file.
      produces:
        - application/zip
      parameters:
      - name: hash
        required: true
        in: path
        description: The hash of the requested file
        type: string
      responses:
        200:
          description: A ZIP file protected with the password 'infected'.
            
        400:
          description: Triggers if no hashes was sent
      
          
          
  /api/v1/file/count:
    get:
      tags:
      - File
      summary: Get the number of files in the database
      produces:
      - application/json
      responses:
        200:
          description: ID of the upload
          schema:
            type: object
            properties:
              count:
                type: number
                description: The count of the files in the database.
        500:
          description: Triggers if the database throws an error
          
  /api/v1/file/upload:
    post:
      tags:
      - File
      summary: Uploads files to the repository
      description: Accepts a single file, or multiple files
      produces:
      - application/json
      consumes:
        - multipart/form-data
      parameters:
        - name: file
          in: formData   # <-----
          description: The uploaded file data
          required: true
          type: file     # <-----
      responses:
        200:
          description: ID of the upload
          schema:
            type: object
            properties:
              id:
                type: string
                description: The upload-id.
        400:
          description: Triggers if no files are sent to the route
          
  /api/v1/file/reanalyze:
    post:
      tags:
      - File
      summary: Triggers reanalysis of files
      description: Triggers new analysis of a specific file using a specific analysis-source.
      produces:
      - application/json
      parameters:
      - in: body
        name: body
        description: Required data to trigger new analysis
        required: true
        schema:
          type: object
          properties:
            analyze_source:
              type: string
              description: The analysis-source to trigger.
              example: virustotal
            hash:
              type: string
              description: The sha256 hash of the file to be analyzed
      responses:
        200:
          description: Analysis triggered successfully
        400:
          description: Triggers if no hash or source was sent
      
  /api/v1/file/download:
    post:
      tags:
      - File
      summary: Download several files
      description: Triggers a download of a multiple files
      produces:
        - application/zip
      parameters:
      - in: body
        name: body
        description: Required data to download files
        required: true
        schema:
          type: object
          properties:
            files:
              type: array
              items:
                type: string
                example: sha256
            password:
              type: string
              example: infected
              description: Wanted password on the ZIP file. Will default to 'infected' if not set
      responses:
        200:
          description: A ZIP file protected with the password 'infected'.
            
        400:
          description: Triggers if no hashes was sent
          
  /api/v1/file/tags:
    put:
      tags:
      - File
      summary: Updating filetags
      description: Lets admins and researcher update the tags of a file
      produces:
      - application/json
      parameters:
      - in: body
        name: body
        description: Required data to download files
        required: true
        schema:
          type: object
          properties:
            hash:
              type: string
              example: "b86850b8c4be8be028c48b50d80bdf8374bbc76d6595956a952318a91835aed9"
              description: The sha256 of the file to be tagged
            tags:
              type: array
              items:
                type: string
                example: exam
      responses:
        200:
          description: Tags are updated.
          schema:
            type: object
            properties: 
              mesage:
                type: string
                example: "Successfully updated tags"
            
        400:
          description: Triggers if no hash or tags was sent
          
        500: 
          description: Triggers if MongoDB is having trouble updating the tags

  /api/v1/user:
    get:
      tags:
      - User
      summary: Get all users
      description: Used for getting all the users. Require admin-access
      produces:
      - application/json
      parameters:
      - name: size
        in: query
        description: The paginate size to get
        required: false
        type: string
      - name: index
        in: query
        description: The paginate index to start from
        required: false
        type: string
      responses:
        200:
          description: Successfull
          schema:
            type: array
            items:
              $ref: '#/definitions/user'
        500:
          description: Internal server error

  /api/v1/user/profile:
    get:
      tags:
      - User
      summary: Authorizing a user
      description: Used for authorizing a user if the user is logged in. Will return user-object if user is serialized.
      produces:
        - application/json
      responses:
        200:
          description: Response containing information of the user logged in.
          schema:
            $ref: '#/definitions/user'
        401:
          description: Triggers if the user is not authenticated
        500: 
          description: Triggers if an internal server error occurs.
    
  /api/v1/user/{id}:
    get:
      tags:
      - User
      summary: Get data about a specific user
      description: Used for grabbing data stored about a user from the database
      produces:
      - application/json
      parameters:
      - name: id
        required: true
        in: path
        description: Feide_id for the user
        type: string

      responses:
        200:
          description: Successful
          schema:
            $ref: '#/definitions/user'
        
        400:
          description: If the request is missing feide_id as parameter
        404:
          description: If the user does not exist in the database
        500:
          description: If the database throws any error
          
    put:
      tags:
      - User
      summary: Edit data for a user
      description: Used for editing data about a user in the database
      produces:
      - application/json
      parameters:
      - name: id
        required: true
        in: path
        description: Feide_id for the user
        type: string
      - in: body
        name: body
        description: Required data to edit user
        required: true
        schema:
          type: object
          properties:
            feide_id:
              type: string
              example: "b86850b8c4be8be028c48b50d80bdf8374bbc76d6595956a952318a91835aed9"
              description: The sha256 of the file to be tagged
            active:
              type: boolean
              example: true
            tags:
              type: array
              items:
                type: string
                example: 'upload'
            role:
              type: string
              example: student
      responses:
        204:
          description: Successfull editing of a user
        400:
          description: Missing parameters
        404:
          description: User does not exist 
        500:
          description: Internal server error
    
    delete:
      tags:
      - User
      summary: Delete user from the database
      description: Used for deleting all data about a user from the database
      produces:
      - application/json
      parameters:
      - name: id
        required: true
        in: path
        description: Feide_id for the user
        type: string
      responses:
        204:
          description: Successfull deleted user
        400:
          description: Missing parameters
        404:
          description: User does not exist 
        500:
          description: Internal server error
        
  /api/v1/user/favorite:
    get:
      tags:
      - User
      summary: Get favorite
      description: Used for getting tags for a user.
      produces:
      - application/json
      parameters:
      - name: size
        in: query
        description: The paginate size to get
        required: false
        type: string
      - name: index
        in: query
        description: The paginate index to start from
        required: false
        type: string
      responses:
        200:
          description: Successfull
        500:
          description: Internal server error
    put:
      tags:
      - User
      summary: Set/remove favorite
      description: Used for adding/removing a favorite for a user. It works as a toggle. If the hash already exist on the user it will remove it, otherwise it will add it.
      produces:
      - application/json
      parameters:
      - in: body
        name: body
        description: Required data to set/remove favorite
        required: true
        schema:
          type: object
          properties:
            hash:
              type: string
              example: "b86850b8c4be8be028c48b50d80bdf8374bbc76d6595956a952318a91835aed9"
      responses:
        200:
          description: Successfull added/removeda favorite
        400:
          description: Missing parameters
        404:
          description: User does not exist
        500:
          description: Internal server error 
  
  /api/v1/user/favorite/count:
    get:
      tags:
      - User
      summary: Get favorite count
      description: Used for getting the number of favorites a user have
      produces:
      - application/json
      responses:
        200:
          description: Successfull
          schema:
            type: object
            properties:
              count:
                type: integer
                example: 32
        500:
          description: Internal server error
  
  
definitions:
  file:
    type: object
    properties:
      pending_analysis:
        type: object
        properties:
          virustotal:
            type: boolean
          malwarbazaar:
            type: boolean
      uploaded_by:
        type: string
      tags:
        type: array
        items:
          type: string
      sha256:
        type: string
      md5:
        type: string
      original_filename:
        type: string
      size:
        type: integer
      date_added:
        type: string
      analyzed_info:
        type: object
        properties:
          peframe:
            type: object
          virustotal:
            type: object
          malwarebazaar:
            type: object
            
  user:
    type: object
    properties:
      feide_id:
        type: string
        example: "5a91dghd2-118b-4e97-8b47-5d2gfhfjg39f"
      name:
        type: string
        example: "Christian Simoes Isnes"
      photo:
        type: string
        example: "https://api.dataporten.no/userinfo/v1/user/media/p:619f1215-5b3c-44cb-8754-e511bc382ec5"
      token:
        type: string
        example: "46h8e4jd-d232-4e56-91e9-3b3ee8365ca4"
      groups:
        type: array
        items:
          type: object
          properties:
            id:
              type: string
              example: "fc:fs:fs:emne:ntnu.no:IMT2021:1"
            type:
              type: string
              example: "fc:fs:emne"
            parent:
              type: string
              example: "fc:org:ntnu.no"
            membership:
              type: object
              properties:
                basic:
                  type: string
                  example: "MEMBER"
                fsroles: 
                  type: array
                  items:
                    type: string
                    example: "STUDENT"
                active:
                  type: boolean
                  example: true
                notafter:
                  type: string
                  example: "2019-12-14T23:00:00Z"
                subjectRelations:
                  type: string
                  example: "undervisning"
                displayName: 
                  type: string
                  example: "Student"
            displayName:
              type: string
              example: "Algoritmiske metoder"
      active:
        type: boolean
        example: true
      authored:
        type: string
        example: true
      role: 
        type: string
        example: "student"
      tags:
        type: array
        items:
          type: string
          example: "upload"
      favorites: 
        type: array
        items:
          type: string
# schemes:
#  - http
# Added by API Auto Mocking Plugin
host: virtserver.swaggerhub.com
basePath: /bachelor_24/MalwareAndGoodwareRepository/1.0.0
schemes:
 - https
 - http