const express = require('express')
var fs = require('fs')
const logger = require('../../logging/infologger')
const path = require('path')
const {
	authorize,
} = require("../../auth/passport-openid-connect/lib/authorize");
const router = express.Router()

module.exports = function (app, express, passport) {

  /**
   * @desc Get logs from the backend
   * @route /api/v1/admin/log
   * @res Stream of logs
   */
  router.ws('/log', (ws, req) => {
    console.log('user', req.user)
    date = new Date()
    //opens todays logfile
    fs.readFile(path.resolve(`logging/infologs/info-${date.getUTCFullYear()}-${('0' +(date.getMonth() + 1)).slice(-2)}-${('0' + date.getDate()).slice(-2)}.log`),'utf8',
      (err, data) => {
        if (err) {
          console.error(err)
        }
        //split the logfile by linebreak to get a valid JSON object
        data.toString().split('\n')
          .forEach((line) => {
            if (line) {
              //send the json object
              ws.send(line)
            }
          })
      })
    //sends the log when a new line is created
    let eventListener = (chunk => {
      JSON.stringify(chunk).split('\n')
        .forEach((line) => {
          if (line) {
            ws.send(line)
          }
        })
    })
    logger.addListener('data', eventListener)
        
    ws.on('close', () => {
      logger.removeListener('data', eventListener)
    })
  })

  /**
   * @desc Get virustotal logs from the backend
   * @route /api/v1/admin/queue/virustotal
   * @res Stream of virustotal logs
   */
  router.ws('/queue/virustotal', (ws, req) => {
    date = new Date()
    //opens todays logfile
    fs.readFile(path.resolve(`logging/infologs/info-${date.getUTCFullYear()}-${('0' +(date.getMonth() + 1)).slice(-2)}-${('0' + date.getDate()).slice(-2)}.log`),'utf8',
      (err, data) => {
        if (err) {
          console.error(err)
        }
        //split the logfile by linebreak to get a valid JSON object
        data.toString().split('\n')
          .forEach((line) => {
            if (line?.message == 'virustotalQueue') {
              //send the json object
              ws.send(line)
            }
          })
      })
    //sends the log when a new line is created
    let eventListener = (chunk => {
      JSON.stringify(chunk).split('\n')
        .forEach((line) => {
          if (JSON.parse(line)?.message == 'virustotalQueue') {
            ws.send(line)
          }
        })
    })
    logger.addListener('data', eventListener)
      
    ws.on('close', () => {
      logger.removeListener('data', eventListener)
    })
  })

  /**
   * @desc Get static analysis logs from the backend
   * @route /api/v1/admin/queue/static
   * @res Stream of static analysis logs
   */
  router.ws('/queue/static', (ws, req) => {
    date = new Date()
    //opens todays logfile
    fs.readFile(path.resolve(`logging/infologs/info-${date.getUTCFullYear()}-${('0' +(date.getMonth() + 1)).slice(-2)}-${('0' + date.getDate()).slice(-2)}.log`),'utf8',
      (err, data) => {
        if (err) {
          console.error(err)
        }
        //split the logfile by linebreak to get a valid JSON object
        data.toString().split('\n')
          .forEach((line) => {
            if (line) {
              if (JSON.parse(line)?.message == 'dockerQueue') {
                //send the json object
                ws.send(line)
              }
            }
          })
      })
    //sends the log when a new line is created
    let eventListener = (chunk => {
      JSON.stringify(chunk).split('\n')
        .forEach((line) => {
          if (JSON.parse(line)?.message == 'dockerQueue') {
            ws.send(line)
          }
        })
    })
    logger.addListener('data', eventListener)
        
    ws.on('close', () => {
      logger.removeListener('data', eventListener)
    })
  })

  return router
}
