const File = require('../models/file')
const fs = require('fs')
const crypto = require('crypto')
var events = require('events')
const { advancedSearchFilter, searchFilter } = require('mongodb-filter-object-parser')

class FileService {
  constructor() {
    this.eventEmitter = new events.EventEmitter()
  }

  async findFile(hash) {
    return new Promise(async (resolve, reject) => {
      File.findOne({ $or: [{ sha256: hash }, { md5: hash }] }, '-_id -__v')
        .lean()
        .then((file) => {
          if (file) {
            resolve(file)
          } else {
            resolve()
          }
        })
    })
  }

  async setPendingAnalysis(hash, source) {
    File.findOneAndUpdate(
      {sha256: hash,}, JSON.parse(`{"pending_analysis.${source}" : true}`)
    ).catch((err) => {
      console.error(err)
    })
  }

  async unsetPendingAnalysis(hash, source) {
    File.findOneAndUpdate(
      { sha256: hash },
      JSON.parse(`{"pending_analysis.${source}" : false}`)
    ).then((ret) => {})
  }

  addAnalysisToDb(hash, source, data) {
    return new Promise(async (resolve, reject) => {
      if (data && data.length > 15000000) {//15 MB
        data = null
      }
      File.findOne({ sha256: hash })
        .then((file) => {
          if (file) {
            let analyzed = JSON.parse(`{"${source}" : ${JSON.stringify(data)}}`)
            if (file.analyzed_info) {
              if (file.analyzed_info[source] && !data) {
                resolve()
              } else {
                var newfile = JSON.parse(`{"analyzed_info" : ${JSON.stringify(
                  Object.assign(file.analyzed_info, analyzed)
                )}}`
                )
              }
            } else {
              var newfile = JSON.parse(
                `{"analyzed_info" : ${JSON.stringify(analyzed)}}`
              )
            }
            //console.log(newfile)
            File.updateOne({ sha256: hash }, newfile).then(() => {
              console.log('added analysis to db')
              resolve()
            })
          } else {
            reject()
          }
        })
        .catch((err) => {
          console.log(err)
        })
    })
  }

  /**
     * @desc Sets the files tags
     * @param $file - Contains the ID of the file to edit
     * @param $tags - Array of tags to be set on the file
     * @return Promise - Resolves result from mongoose, or error if reject
    */
   tags(file, tags){
    return File.findOneAndUpdate({ sha256 : file }, { tags: tags }, {new: true})
  }

  /**
   * @desc Get files in db
   * @param count - number of files,
   * @param skip - how many to skip from top, defaults to 0
   * @return Promise - Array of objects containing files
   */

  getFiles(count = 100, skip = 0, search = undefined) {
    let simpleFilter = {}
    if (search) {
      simpleFilter = searchFilter(search)
    }
    
    console.log(simpleFilter)
    //console.log(search)
    //, '-_id -__v'
    return File.find(simpleFilter)
      .lean()
      .limit(Number(count))
      .skip(Number(skip) * Number(count))
  }

  /**
   * @desc Get number of files in db
   * @return Promise - number of documents in the file collection
   */

  getFileCount() {
    return new Promise(async function (resolve, reject) {
      File.countDocuments(function (err, count) {
        if (err) {
         reject(err) 
        } else {
          resolve(count)
        }
      })
    })
  }

  /**
   * @desc create new file
   * @param array $data - array containing required data for the file
   * @return object - Returns the db object
   */
  async createNew(data) {
    const file = new File(data)

    return file.save()
  }

  async getDbFilePath(hash) {
    return new Promise(async function (resolve, reject) {
      // get first two characters of hash
      let folder1 = hash.slice(0, 2)
      //get next two characters of hash
      let folder2 = hash.slice(2, 4)
      //concatenate to get the full path to store the file
      let folderpath = `${process.env.dbfilePath}/${folder1}/${folder2}`
      //create directory and return it
      fs.mkdir(folderpath, { recursive: true }, (err => {
        if (err) {
          console.error(err)
        } else {
          resolve(`${process.env.dbfilePath}/${folder1}/${folder2}/${hash}`)
        }
      }))
    })
  }

  searchFile(params) {
    const simpleFilter = searchFilter(params)
    const textFilter = textSearchFilter(params, ['color', 'height'])
    const advancedFilter = advancedSearchFilter(string)

    const query1 = file.find(simpleFilter)
    const query2 = file.find(textFilter)
    const query3 = file.find(advancedFilter)
  }

  encryption(inPath, outPath, mode) {
    return new Promise(async function (resolve, reject) {
      let cipher
      var iv = 'fb1f4b0a7daaada6cae678df32fad0f0'.toString('hex').slice(0, 16)
      var key = '361afbb90589c1dbaba30c9c8b380772449df5b01544e084fe473b501f583129'
        .toString('hex')
        .slice(0, 32)

      if (mode === 'encrypt') {
        cipher = crypto.createCipheriv('aes-256-cbc', key, iv)
      } else if (mode === 'decrypt') {
        cipher = crypto.createDecipheriv('aes-256-cbc', key, iv)
      } else {
        reject()
      }

      try {
        var input = fs.createReadStream(inPath)
        var output = fs.createWriteStream(outPath)
      } catch (err) {
        console.error(err)
      }
      input.pipe(cipher).pipe(output)

      input.on('error', async function (err) {
        console.error(err)
        resolve()
      })
      output.on('error', async function (err) {
        console.error(err)
        resolve()
      })
      output.on('finish', async function () {
        console.log(mode + 'ion finished!')
        resolve()
      })

    })
  }

  uploadStatus() {
    return this.eventEmitter
  }
}

module.exports = new FileService()
