const { readableObjectMode } = require('../logging/infologger')
const User = require('../models/user')

class UserService {

    /**
     * @desc Create a new user
     * @param json-object $data - Contains username and password
     * @return Promise - Resolves result from mongoose, or error if reject
    */ 
    createUser(data){
        const user = new User({
            feide_id: data.data.sub,
            name: data.data.name,
            password: data.password,
            role: "student",
            lastLogin: Date.now()
        })
        return user.save()
    }

    /**
     * @desc Get number of files in db
     * @return Promise - number of documents in the file collection
    */  

  favorite(feide) {
    return new Promise(function(resolve, reject) {
      User.findOne({ "feide_id" : feide })
        .then(user => {
            resolve(user.favorites)
        })
        .catch(error => {
            reject(error)
        })
    })
  }

    /**
     * @desc Edit a user
     * @param json-object $data - Contains updated data
     * @param string $feide - Contains ID used to identify user
     * @return Promise - Resolves result from mongoose, or error if reject
    */
  editUser(data, feide) {
    return User.findOneAndUpdate({ "feide_id" : feide }, { active: data.active, tags: data.tags, role: data.role }, {new: true})
  }

    /**
     * @desc Changes favorite of a user
     * @param array $data - Contains updated data
     * @param string $feide - Contains ID used to identify user
     * @return Promise - Resolves result from mongoose, or error if reject
    */
    changeFavorite(data, feide) {
        return User.findOneAndUpdate({ "feide_id" : feide }, { favorites: data }, {new: true})
    }

  
  
    /**
     * @desc Returns a specific user
     * @param json-object $data - Contains username or _id
     * @return Promise - Resolves result from mongoose, or error if reject
    */
    findUser(data){
        return User.findOne({ "feide_id" : data })
    }

    /**
     * @desc Update last login time of a user
     * @param json-object $user - Contains username of the user to update
     * @return Promise - Resolves result from mongoose, or error if reject
    */
    setlastlogin(id){
        return User.findOneAndUpdate({'feide_id' : id}, {'lastLogin': Date.now()})
    }

    /**
     * @desc Get all users from the db
     * @return Promise - Resolves result from mongoose, or error if reject
    */
    getUsers(count, skip = 0){
        return User.find().limit(count).skip(skip * count)
    }

    /**
     * @desc Delete single user
     * @param json-object $data - Contains username or _id of user to be deleted
     * @return Promise - Resolves result from mongoose, or error if reject
    */
    deleteUser(data){
        return User.deleteOne({ "feide_id" : data })
    }

    /**
     * @desc Delete multiple users
     * @param json-object $data - Contains array of _id's to be deleted. !MUST BE ARRAY, else boom
     * @return Promise - Resolves result from mongoose, or error if reject
    */
    deleteUsers(data){
        return User.deleteMany({ "feide_id" : data})
    }

    getUserCount(){
        return new Promise(async function(resolve, reject) {
            User.countDocuments(function(err, count) {
                resolve(count)
            })
        })
    }
}

module.exports = new UserService()