const mongoose = require('mongoose')
const uniqueValidator = require("mongoose-unique-validator")
require('mongoose-long')(mongoose);
const {Types: {Long}} = mongoose;

const fileSchema = new mongoose.Schema({
    sha256: {
        type: String, 
        required: true, 
        unique: true, 
        immutable: true
    },
    md5: {
        type: String, 
        required: true, 
        unique: true, 
        immutable: true
    },
    original_filename: {
        type: String,  
        required: true,
        index: true
    },
    size: {
        type: Long,
        required: true
    }, 
    filetype: {
        type: String,  
        default:'Unknown',
        index: true
    }, 
    date_added: {
        type: Date, 
        default: Date.now, 
        index: true
    },
    uploaded_by: {
      type: String, 
      index: true,
      default: 'System'
    },
    pending_analysis: {
        virustotal: {type: Boolean,default: false},
        static_analysis: { type: Boolean, default: false }
        },
    analyzed_info: {},
    tags: {
        type: [String]
    }
})

fileSchema.plugin(uniqueValidator)
fileSchema.index({ sha256: 'text', md5: 'text', original_filename: 'text', size: 'text', filetype: 'text', uploaded_by: 'text', tags: 'text' })



module.exports = mongoose.model('file', fileSchema)