const assert = require('assert')
const file = require('../../models/file')



function create_testfile(){
    return testfile = new file({
        sha256 : '361afbb90589c1dbaba30c9c8b380772449df5b01544e084fe473b501f583129',
        md5 : '51c76e05546a8006277e906e9a7970d5',
        original_filename : 'emotet.doc',
        size : '583579', 
        filetype : 'docx', 
    })
}

describe('Database unit tests', () => {

    

    it('Saves a file to the database', (done) => {
        
        var testfile = create_testfile()

        testfile.save().then(() => {
            assert(testfile.isNew === false)
            done()
        })
    })

    it('Finds one record from the database', (done) => {

        var testfile = create_testfile()

        testfile.save().then(() => {
            file.findOne({md5:'51c76e05546a8006277e906e9a7970d5'}).then((result) => {
                assert(result.md5 === '51c76e05546a8006277e906e9a7970d5')
                done()
            })
        })
    })

    it('Finds a record by ID', (done) => {

        var testfile = create_testfile()

        testfile.save().then(() => {
            file.findOne({_id: testfile._id}).then((result) => {
                assert(result._id.toString() === testfile._id.toString())
                done()
            })
        })
    })
    
    it('Deletes a record from the database', (done) => {

        var testfile = create_testfile()

        testfile.save().then(() => {
            file.findOneAndRemove({md5:'51c76e05546a8006277e906e9a7970d5'}).then(() => {
                file.findOne({md5:'51c76e05546a8006277e906e9a7970d5'}).then((result) => {
                    assert (result === null)
                    done()
                })
            })
        })
    })

    it('Updates a record in the database', (done) => {

        var testfile = create_testfile()

        testfile.save().then(() => {
            file.findOneAndUpdate({original_filename:'emotet.doc'}, {original_filename:'pdf.exe'}).then(() => {
                file.findOne({_id: testfile._id}).then((result) => {
                    assert(result.original_filename === 'pdf.exe')
                    done()   
                })
            })
        })
    })
    
    it.skip('Tries to update md5 hash, this test should fail', (done) => {
        
        var testfile = create_testfile()

        testfile.save().then(() => {
            file.findOneAndUpdate({md5:'51c76e05546a8006277e906e9a7970d5'}, {md5:'2f4b6313a5b73e0e3b0191cd43832a9a'}).then(() => {
                file.findOne({_id: testfile._id}).then((result) => {
                    assert.fail(result.md5 === '2f4b6313a5b73e0e3b0191cd43832a9a')
                    done()         
                })
            })
        })
    })

    it ('Created an index with sub-documents', (done) => {

        var nestedTest = new file({
            sha256: '361afbb90589c1dbaba30c9c8b380772449df5b01544e084fe473b501f583129',
            md5: '51c76e05546a8006277e906e9a7970d5',
            original_filename: 'emotet.doc',
            size: '583579', 
            filetype: 'docx', 
            tags: [{tag: 'malware'}]
        })

        nestedTest.save().then(() => {
            file.findOne({md5: '51c76e05546a8006277e906e9a7970d5'}).then((record) => {
                console.log(record)
                assert(record.tags.tag === 'malware')
                done()
            })
        }).catch(done)
    })
})